C---(SOM3DX.f)---simple test of SOM advection subroutine: qvect.f  (mp, 3/07)
c
c  run with <stdin.som and look at stdout and fort.16 (saved here)
c
c (1) moves a 6x6 block of mix.ratio 100 across a 3-D grid with background = 0
c (2) moves a 6x6 hole of mix.ratio 100 across a 3-D grid with background = 200
c
c Compile together with QVECT.f and run interactively (see stdin.dat,stdout.dat)

      implicit none

      real*8  QM(99),     ! air mass in box at start, !**=REDEFINED after flow
     &    QU(100),QV(100),QW(100)
      real*8  QTT(99)  !** tracer mass in box [I]
      real*8  QXT(99),QXX(99),    !** 1st & 2nd moments of tracer in X-direct
     &        QXY(99),QXZ(99),    !** coupled 1st moments including X component
     &        QYZ(99),QYT(99),QYY(99),QZT(99),QZZ(99) !** other SOMs without X
      integer LIMTR
      integer I,J,K, IQ,JQ,KQ,NQ, ITER,NITER, NCFL,ICFL, NT
      real*8  ALFA,BETA,GAMA, QD(99), ERRMAX,ERRMIN,ERRRMS,ERRABS,ERR
      real*8 STT(24,24,24),SAD(24,24,24)          ,QSUM(24,24,24)
      real*8 SUT(24,24,24),SUU(24,24,24),SUV(24,24,24)
      real*8 SVT(24,24,24),SVV(24,24,24),SVW(24,24,24)
      real*8 SWT(24,24,24),SWW(24,24,24),SUW(24,24,24)
C-----------------------------------------------------------------------
      open(16,access='APPEND')
   11 write(6,*) ' enter: LIMTR, STEPX=1/NCFL, NITER'
      read(5,*)  LIMTR,NCFL,NITER

      if (LIMTR .lt. 0) goto 99

      write( 6,*) ' LIMTR/NCFL/NITER:', LIMTR,NCFL,NITER
      write(16,*) ' LIMTR/NCFL/NITER:', LIMTR,NCFL,NITER

C---set up advective flows, GAMA=1/NCFL, BETA=GAMA/2, ALFA=GAMA/2
      GAMA = 1.D0/FLOAT(NCFL)
      BETA = GAMA/2.d0
      ALFA = GAMA/2.d0
      NQ = 24
      do K=1,NQ
      do J=1,NQ
      do I=1,NQ
        QSUM(I,J,K) = 0.D0
      enddo
      enddo
      enddo

      do 40 NT=1,2

      do K=1,NQ
      do J=1,NQ
      do I=1,NQ
        SAD(I,J,K) = 1.D0
        if (NT.eq.1) then 
          STT(I,J,K) =   0.0
        ELSE 
          STT(I,J,K) = 200.0
        endif 
        SUT(I,J,K) = 0.0
        SUU(I,J,K) = 0.0
        SUV(I,J,K) = 0.0
        SVT(I,J,K) = 0.0
        SVV(I,J,K) = 0.0
        SVW(I,J,K) = 0.0
        SWT(I,J,K) = 0.0
        SWW(I,J,K) = 0.0
        SUW(I,J,K) = 0.0
      enddo
      enddo
      enddo

      do K=1,6
      do J=1,6
      do I=1,6
        STT(I,J,K) = 100.0
      enddo
      enddo
      enddo

      do K=1,NQ+1
      do J=1,NQ+1
      do I=1,NQ+1
        QU(I) = ALFA
        QV(I) = BETA
        QW(I) = GAMA
      enddo
      enddo
      enddo

C----diagnostics:
       if (NT .eq. 1) then
        write( 6,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') 0,0.,100.,0.0,0.0
        write(16,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') 0,0.,100.,0.0,0.0
       else
        write( 6,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') 0,100.,200.,0.0,0.0
        write(16,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') 0,100.,200.,0.0,0.0
       endif

C----major loop: NITER
      do ITER=1,NITER

C----for each grid-step (ITER) do NFCL sub-steps: V-U-V
        do ICFL=1,NCFL

C----STEP: U
         do K=1,NQ
         do J=1,NQ
          do I=1,NQ
           QM(I) = SAD(I,J,K)
           QTT(I) = STT(I,J,K)
           QXT(I) = SUT(I,J,K)
           QYT(I) = SVT(I,J,K)
           QXX(I) = SUU(I,J,K)
           QYY(I) = SVV(I,J,K)
           QXY(I) = SUV(I,J,K)
           QZT(I) = SWT(I,J,K)
           QZZ(I) = SWW(I,J,K)
           QXZ(I) = SUW(I,J,K)
           QYZ(I) = SVW(I,J,K)
          enddo   
          call QVECT (QM,QU,NQ,LIMTR,
     &      QTT,QXT,QXX,QXY,QXZ,QYT,QYY,QZT,QZZ,QYZ)
          do I=1,NQ
           SAD(I,J,K) = QM(I)
           STT(I,J,K) = QTT(I)
           SUT(I,J,K) = QXT(I)
           SVT(I,J,K) = QYT(I)
           SUU(I,J,K) = QXX(I)
           SVV(I,J,K) = QYY(I) 
           SUV(I,J,K) = QXY(I) 
           SWT(I,J,K) = QZT(I)
           SWW(I,J,K) = QZZ(I)
           SUW(I,J,K) = QXZ(I)
           SVW(I,J,K) = QYZ(I)
          enddo   
         enddo
         enddo

C----STEP: V
         do K=1,NQ
         do I=1,NQ
          do J=1,NQ
           QM(J) = SAD(I,J,K)
           QTT(J) = STT(I,J,K)
           QXT(J) = SUT(I,J,K)
           QYT(J) = SVT(I,J,K)
           QXX(J) = SUU(I,J,K)
           QYY(J) = SVV(I,J,K)
           QXY(J) = SUV(I,J,K)
           QZT(J) = SWT(I,J,K)
           QZZ(J) = SWW(I,J,K)
           QXZ(J) = SUW(I,J,K)
           QYZ(J) = SVW(I,J,K)
          enddo   
          call QVECT (QM,QV,NQ,LIMTR,
     &      QTT,QYT,QYY,QXY,QYZ,QXT,QXX,QZT,QZZ,QXZ)
          do J=1,NQ
           SAD(I,J,K) = QM(J)
           STT(I,J,K) = QTT(J)
           SUT(I,J,K) = QXT(J)
           SVT(I,J,K) = QYT(J)
           SUU(I,J,K) = QXX(J)
           SVV(I,J,K) = QYY(J) 
           SUV(I,J,K) = QXY(J) 
           SWT(I,J,K) = QZT(J)
           SWW(I,J,K) = QZZ(J)
           SUW(I,J,K) = QXZ(J)
           SVW(I,J,K) = QYZ(J)
          enddo   
         enddo
         enddo

C----STEP: W
         do J=1,NQ
         do I=1,NQ
          do K=1,NQ
           QM(K) = SAD(I,J,K)
           QTT(K) = STT(I,J,K)
           QXT(K) = SUT(I,J,K)
           QYT(K) = SVT(I,J,K)
           QXX(K) = SUU(I,J,K)
           QYY(K) = SVV(I,J,K)
           QXY(K) = SUV(I,J,K)
           QZT(K) = SWT(I,J,K)
           QZZ(K) = SWW(I,J,K)
           QXZ(K) = SUW(I,J,K)
           QYZ(K) = SVW(I,J,K)
          enddo   
          call QVECT (QM,QW,NQ,LIMTR,
     &      QTT,QZT,QZZ,QXZ,QYZ,QYT,QYY,QXT,QXX,QXY)
          do K=1,NQ
           SAD(I,J,K) = QM(K)
           STT(I,J,K) = QTT(K)
           SUT(I,J,K) = QXT(K)
           SVT(I,J,K) = QYT(K)
           SUU(I,J,K) = QXX(K)
           SVV(I,J,K) = QYY(K) 
           SUV(I,J,K) = QXY(K) 
           SWT(I,J,K) = QZT(K)
           SWW(I,J,K) = QZZ(K)
           SUW(I,J,K) = QXZ(K)
           SVW(I,J,K) = QYZ(K)
          enddo   
         enddo
         enddo

C----STEP: V
         do K=1,NQ
         do I=1,NQ
          do J=1,NQ
           QM(J) = SAD(I,J,K)
           QTT(J) = STT(I,J,K)
           QXT(J) = SUT(I,J,K)
           QYT(J) = SVT(I,J,K)
           QXX(J) = SUU(I,J,K)
           QYY(J) = SVV(I,J,K)
           QXY(J) = SUV(I,J,K)
           QZT(J) = SWT(I,J,K)
           QZZ(J) = SWW(I,J,K)
           QXZ(J) = SUW(I,J,K)
           QYZ(J) = SVW(I,J,K)
          enddo   
          call QVECT (QM,QV,NQ,LIMTR,
     &      QTT,QYT,QYY,QXY,QYZ,QXT,QXX,QZT,QZZ,QXZ)
          do J=1,NQ
           SAD(I,J,K) = QM(J)
           STT(I,J,K) = QTT(J)
           SUT(I,J,K) = QXT(J)
           SVT(I,J,K) = QYT(J)
           SUU(I,J,K) = QXX(J)
           SVV(I,J,K) = QYY(J) 
           SUV(I,J,K) = QXY(J) 
           SWT(I,J,K) = QZT(J)
           SWW(I,J,K) = QZZ(J)
           SUW(I,J,K) = QXZ(J)
           SVW(I,J,K) = QYZ(J)
          enddo   
         enddo
         enddo

C----STEP: U
         do K=1,NQ
         do J=1,NQ
          do I=1,NQ
           QM(I) = SAD(I,J,K)
           QTT(I) = STT(I,J,K)
           QXT(I) = SUT(I,J,K)
           QYT(I) = SVT(I,J,K)
           QXX(I) = SUU(I,J,K)
           QYY(I) = SVV(I,J,K)
           QXY(I) = SUV(I,J,K)
           QZT(I) = SWT(I,J,K)
           QZZ(I) = SWW(I,J,K)
           QXZ(I) = SUW(I,J,K)
           QYZ(I) = SVW(I,J,K)
          enddo   
          call QVECT (QM,QU,NQ,LIMTR,
     &      QTT,QXT,QXX,QXY,QXZ,QYT,QYY,QZT,QZZ,QYZ)
          do I=1,NQ
           SAD(I,J,K) = QM(I)
           STT(I,J,K) = QTT(I)
           SUT(I,J,K) = QXT(I)
           SVT(I,J,K) = QYT(I)
           SUU(I,J,K) = QXX(I)
           SVV(I,J,K) = QYY(I) 
           SUV(I,J,K) = QXY(I) 
           SWT(I,J,K) = QZT(I)
           SWW(I,J,K) = QZZ(I)
           SUW(I,J,K) = QXZ(I)
           SVW(I,J,K) = QYZ(I)
          enddo   
         enddo
         enddo
C------------------------------------------------------

        enddo
C----finished one CFL cycle

          ERRABS = 0.0
          ERRRMS = 0.0
          ERRMAX = 0.0
          ERRMIN = 999999.
         do KQ=1,NQ
          if (ITER .eq. NITER) write(16,'(1X,A2,I3)') 'K=',KQ
          K = MOD(KQ-ITER+200*NQ-1 ,NQ) + 1
          do JQ=1,NQ
           J = MOD(JQ-ITER+200*NQ-1 ,NQ) + 1
           do IQ=1,NQ
            I = MOD(IQ-ITER+200*NQ-1 ,NQ) + 1
            QD(I) = STT(IQ,JQ,KQ)/SAD(IQ,JQ,KQ)
           enddo
           do I=1,NQ  
            if (NT.eq.1) then 
             ERR = QD(I)
             if (I.le.6 .and. J.le.6 .and. K.le.6) ERR = QD(I)-100.D0
            else 
             ERR = QD(I) - 200.D0
             if (I.le.6 .and. J.le.6 .and. K.le.6) ERR = QD(I)-100.D0
            endif
             ERRABS = max(ERRABS, abs(ERR))
             ERRRMS = ERRRMS + ERR*ERR
             ERRMAX = max(ERRMAX, QD(I))
             ERRMIN = min(ERRMIN, QD(I))
           enddo
C----can limit 3D print to neighbor: if (J.eq.3 .and. (I.gt.22 .or. I.lt.9)) then
            if (ITER .eq. NITER) then
              write(16,'(1X,I3,32F6.1)') J,(QD(I), I=1,NQ)
            endif
          enddo
        enddo
             ERRRMS =  sqrt(ERRRMS/216.d0)

         write( 6,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') ITER,
     &            ERRMIN,ERRMAX,ERRABS,ERRRMS 
        if (ITER .eq. NITER) then
         write(16,'(15H Mn/Mx/abs/rms:,I4,4F7.2)') ITER,
     &            ERRMIN,ERRMAX,ERRABS,ERRRMS        
        endif


      enddo

c---now sum the +100 and -100 (hole) experiments to check for symm errors
        do K=1,NQ
        do J=1,NQ
        do I=1,NQ
          QSUM(I,J,K) = QSUM(I,J,K) + STT(I,J,K)/SAD(I,J,K)
        enddo
        enddo
        enddo

   40 continue

          ERRABS = 0.0
          ERRRMS = 0.0
          ERRMAX = 0.0
          ERRMIN = 999999.
        do I=1,NQ
         do J=1,NQ
          do K=1,NQ
            ERR = QSUM(I,J,K)-200.D0
            ERRABS = max(ERRABS, abs(ERR))
            ERRRMS = ERRRMS + ERR*ERR
            ERRMAX = max(ERRMAX, QSUM(I,J,K))
            ERRMIN = min(ERRMIN, QSUM(I,J,K))
          enddo
         enddo
        enddo
        ERRRMS =  sqrt(ERRRMS/216.d0)


      write( 6,'(11H QSUM 1 + 2,8x,4F7.2)') ERRMIN,ERRMAX,ERRABS,ERRRMS
      write(16,'(11H QSUM 1 + 2,8x,4F7.2)') ERRMIN,ERRMAX,ERRABS,ERRRMS
      do K=1,NQ
       write(16,'(1X,A3,I3)') 'Kq=',K
       do J=1,NQ
        write(16,'(1X,I3,32F6.1)') J,(QSUM(I,J,K), I=1,NQ)
       enddo
      enddo

      goto 11
   99 continue
   
      stop
      end

